from airflow import DAG
from airflow.providers.apache.spark.operators.spark_submit import SparkSubmitOperator
from airflow.operators.python import BranchPythonOperator
from airflow.operators.empty import EmptyOperator
from airflow.providers.http.operators.http import SimpleHttpOperator
from datetime import timedelta
import os
import glob

default_args = {
    'owner': 'airflow',
    'depends_on_past': False,
    'retries': 1,
    'retry_delay': timedelta(minutes=5),
}

dag = DAG(
    'telco_customer_churn_data_pipeline', 
    default_args=default_args,
    description='DAG to clean Telco Customer Churn dataset',
    schedule_interval=None,
    catchup=False,
)

spark_clean = SparkSubmitOperator(
    task_id='spark_data_cleaning',
    application='/opt/spark/jobs/clean.py',
    name='telco_customer_churn_cleaning',
    conn_id='spark_default',
    dag=dag,
)

spark_preprocess = SparkSubmitOperator(
    task_id='spark_data_preprocessing',
    application='/opt/spark/jobs/preprocess.py',
    name='telco_customer_churn_preprocessing',
    conn_id='spark_default',
    dag=dag,
)

spark_drift_check = SparkSubmitOperator(
    task_id='spark_data_drift_check',
    application='/opt/spark/jobs/drift_check.py',
    name='telco_customer_churn_drift_check',
    conn_id='spark_default',
    dag=dag,
)

def decide_next_task():
    base_path = "/opt/data/versioning/"
    base_name = "clean"
    
    existing_files = glob.glob(f"{base_path}{base_name}*.csv")
    
    version_numbers = []
    for file in existing_files:
        parts = file.replace(base_path, "").replace(".csv", "").replace(base_name, "")
        if parts.isdigit():
            version_numbers.append(int(parts))
    
    version_numbers.sort()
    
    if len(version_numbers) <= 1:
        return "train_model"
    
    drift_result_path = "/opt/data/drift_result.txt"
    if os.path.exists(drift_result_path):
        with open(drift_result_path, "r") as f:
            drift_flag = f.read().strip()
        if drift_flag == "1":
            return "train_model"
    return "no_drift"

branch_task = BranchPythonOperator(
    task_id='check_drift_result',
    python_callable=decide_next_task,
    dag=dag,
)

train_model = SparkSubmitOperator(
    task_id='train_model',
    application='/opt/spark/jobs/train.py',
    name='model_training',
    conn_id='spark_default',
    dag=dag,
)

serve_model = SimpleHttpOperator(
    task_id='serve_model',
    http_conn_id='mlflow_server',
    endpoint='serve_latest',
    method='POST',
    headers={'Content-Type': 'application/json'},
    data='{}',
    dag=dag,
)

no_drift = EmptyOperator(
    task_id='no_drift',
    dag=dag,
)

spark_clean >> spark_preprocess >> spark_drift_check >> branch_task
branch_task >> train_model >> serve_model
branch_task >> no_drift
