from pyspark.sql import SparkSession
from pyspark.sql import functions as F
from pyspark.sql.functions import col, when
import glob
import os
import re
import shutil

spark = SparkSession.builder \
    .appName("TelcoCustomerChurn") \
    .getOrCreate()

df = spark.read.csv("/opt/data/TelcoCustomerChurn.csv", header=True, inferSchema=True)

df = df.drop("customerID")

df = df.withColumn("TotalCharges", col("TotalCharges").cast("double"))

df = df.withColumn(
    "TotalCharges",
    F.when(F.col("tenure") == 0, 0).otherwise(F.col("TotalCharges"))
)

percentile = df.filter(F.col("tenure") != 0).approxQuantile("TotalCharges", [0.5], 0.01)
median_total_charges = percentile[0]

df = df.fillna({"TotalCharges": median_total_charges})

columns_to_replace = [
    "MultipleLines", "OnlineSecurity", "OnlineBackup",
    "DeviceProtection", "TechSupport", "StreamingTV", "StreamingMovies"
]

for col_name in columns_to_replace:
    df = df.withColumn(
        col_name,
        when(col(col_name).isin(["No phone service", "No internet service"]), "No")
        .otherwise(col(col_name))
    )

df = df.withColumn(
    'SeniorCitizen',
    when(col('SeniorCitizen') == 0, 'No')
    .when(col('SeniorCitizen') == 1, 'Yes')
    .otherwise(col('SeniorCitizen'))
)

column_list = df.columns

def to_snake_case(name):
    return re.sub('([a-z0-9])([A-Z])', r'\1_\2', name).lower()

new_columns = [to_snake_case(col) for col in column_list]

for old_col, new_col in zip(df.columns, new_columns):
    df = df.withColumnRenamed(old_col, new_col)

output_dir = "/opt/data/clean_temp"
base_file_path = "/opt/data/versioning/"
base_file_name = "clean"

def get_next_version(base_path, base_name):
    existing_files = glob.glob(f"{base_path}{base_name}*.csv")
    
    version_numbers = []
    for file in existing_files:
        parts = file.replace(base_path, "").replace(".csv", "").replace(base_name, "")
        if parts.isdigit():
            version_numbers.append(int(parts))
    
    return max(version_numbers, default=0) + 1

next_version = get_next_version(base_file_path, base_file_name)

if os.path.exists(output_dir):
    shutil.rmtree(output_dir)

df.coalesce(1).write.option("header", "true").csv(output_dir)

if not os.path.exists(base_file_path):
    os.makedirs(base_file_path)

for file in os.listdir(output_dir):
    if file.startswith("part-"):
        final_file = f"{base_file_path}{base_file_name}{next_version}.csv"
        shutil.move(os.path.join(output_dir, file), final_file)

shutil.rmtree(output_dir)
