import pandas as pd
import numpy as np
import glob
from pyspark.sql import SparkSession

spark = SparkSession.builder \
    .appName("DataDriftCheck") \
    .getOrCreate()

def get_latest_versions(base_path, base_name):
    existing_files = glob.glob(f"{base_path}{base_name}*.csv")
    version_numbers = []
    for file in existing_files:
        parts = file.replace(base_path, "").replace(".csv", "").replace(base_name, "")
        if parts.isdigit():
            version_numbers.append(int(parts))
    version_numbers.sort()
    if len(version_numbers) < 2:
        return None, None
    return version_numbers[-1], version_numbers[-2]

def calculate_psi(base_distribution, new_distribution, bins=10):
    psi = 0
    for base_bin, new_bin in zip(base_distribution, new_distribution):
        if base_bin > 0 and new_bin > 0:
            psi += (new_bin - base_bin) * np.log(new_bin / base_bin)
    return psi

def calculate_column_psi(latest_col, previous_col, bins=10):
    bin_edges = pd.cut(pd.concat([latest_col, previous_col]), bins=bins, retbins=True)[1]

    latest_distribution = pd.cut(latest_col, bins=bin_edges).value_counts(normalize=True, sort=False)
    previous_distribution = pd.cut(previous_col, bins=bin_edges).value_counts(normalize=True, sort=False)
    
    return calculate_psi(previous_distribution, latest_distribution)

base_path = "/opt/data/versioning/"
base_name = "clean"
drift_result_file = "/opt/data/drift_result.txt"

latest_version, previous_version = get_latest_versions(base_path, base_name)

if latest_version is None or previous_version is None:
    with open(drift_result_file, "w") as f:
        f.write("1")
    print("Drift check completed. Drift detected: True")
    
else:
    latest_file = f"{base_path}{base_name}{latest_version}.csv"
    previous_file = f"{base_path}{base_name}{previous_version}.csv"
    
    latest_df = pd.read_csv(latest_file)
    previous_df = pd.read_csv(previous_file)

    drift_detected = False
    psi_threshold = 0.1

    for column in latest_df.select_dtypes(include=['number']).columns:
        latest_col = latest_df[column].dropna()
        previous_col = previous_df[column].dropna()

        if not latest_col.empty and not previous_col.empty:
            column_psi = calculate_column_psi(latest_col, previous_col)
            print(f"PSI for column '{column}': {column_psi}")

            if column_psi > psi_threshold:
                drift_detected = True
                break

    with open(drift_result_file, "w") as f:
        f.write("1" if drift_detected else "0")
    print("Drift check completed. Drift detected:", drift_detected)
