from imblearn.over_sampling import SMOTE
from pyspark.sql import SparkSession
from pyspark.sql.types import NumericType, FloatType
from pyspark.sql.functions import udf, col
from pyspark.ml.feature import (
    MinMaxScaler,
    VectorAssembler,
    StringIndexer,
    OneHotEncoder,
    VectorSlicer
)
from pyspark.ml.functions import vector_to_array
from pyspark.ml import Pipeline
import glob
import os
import pandas as pd
import shutil

spark = SparkSession.builder \
    .appName("Preprocessing") \
    .getOrCreate()

base_path = "/opt/data/versioning/"
base_clean_name = "clean"
base_train_name = "train_data"
base_test_name = "test_data"

def get_highest_version(base_path, base_name):
    existing_files = glob.glob(f"{base_path}{base_name}*.csv")
    
    version_numbers = []
    for file in existing_files:
        parts = file.replace(base_path, "").replace(".csv", "").replace(base_name, "")
        if parts.isdigit():
            version_numbers.append(int(parts))
    
    return max(version_numbers, default=1)

highest_version = get_highest_version(base_path, base_clean_name)

clean_file = f"{base_path}{base_clean_name}{highest_version}.csv"
train_file = f"{base_train_name}{highest_version}"
test_file = f"{base_test_name}{highest_version}"

df = spark.read.csv(clean_file, header=True, inferSchema=True)

column_numerical = []
column_categorical = []

for col_name, col_type in df.dtypes:
    if isinstance(df.schema[col_name].dataType, NumericType):
        column_numerical.append(col_name)
    else:
        column_categorical.append(col_name)

column_categorical.remove("churn")

data_train, data_test = df.randomSplit([0.7, 0.3], seed=42)

indexers = [StringIndexer(inputCol=col, outputCol=f"{col}_index") for col in column_categorical + ["churn"]]
index_features = [f"{col}_index" for col in column_categorical]

encoders = [
    OneHotEncoder(inputCol=f"{col}_index", outputCol=f"{col}_encoded", dropLast=True) for col in column_categorical
]

numerical_assembler = VectorAssembler(inputCols=column_numerical, outputCol="numerical_features")

scaler = MinMaxScaler(inputCol="numerical_features", outputCol="scaled_numerical_features")

slicers = [
    VectorSlicer(inputCol="scaled_numerical_features", outputCol=f"scaled_{col}", indices=[i])
    for i, col in enumerate(column_numerical)
]

pipeline = Pipeline(stages=indexers + encoders + [numerical_assembler, scaler] + slicers)

pipeline_fit = pipeline.fit(data_train)

data_train_processed = pipeline_fit.transform(data_train)
data_test_processed = pipeline_fit.transform(data_test)

def vector_to_float(vector):
    return float(vector[0]) if vector else None
vector_to_float_udf = udf(vector_to_float, FloatType())

def postprocess_data(data):
    encoded_features = []
    for col_name in column_categorical:
        encoded_col = f"{col_name}_encoded"
        
        data = data.withColumn(f"{encoded_col}_array", vector_to_array(col(encoded_col)))
        
        size = data.select(f"{encoded_col}_array").first()[0]
        
        for i in range(len(size)):
            new_col_name = f"{col_name}_category_{i}"
            data = data.withColumn(new_col_name, col(f"{encoded_col}_array")[i])
            encoded_features.append(f"{col_name}_category_{i}")
    
    for col_name in column_numerical:
        data = data.withColumn(f"scaled_{col_name}_tmp", vector_to_float_udf(f"scaled_{col_name}"))
        data = data.drop(f"scaled_{col_name}").withColumnRenamed(f"scaled_{col_name}_tmp", f"scaled_{col_name}")

    scaled_features = [f"scaled_{col}" for col in column_numerical]
    all_columns = scaled_features + encoded_features + ["churn_index"]
    data = data.select(*all_columns).withColumnRenamed("churn_index", "churn")
    
    return data

data_train_processed = postprocess_data(data_train_processed)
data_test_processed = postprocess_data(data_test_processed)

pandas_df_train = data_train_processed.toPandas()
X_train = pandas_df_train.drop(columns=["churn"])
Y_train = pandas_df_train["churn"]
smote = SMOTE(random_state=1)
X_train_smote, y_train_smote = smote.fit_resample(X_train, Y_train)
X_train_smote_df = pd.DataFrame(X_train_smote, columns=X_train.columns)
y_train_smote_df = pd.DataFrame(y_train_smote, columns=['churn'])
data_smote = pd.concat([X_train_smote_df, y_train_smote_df], axis=1)
data_train_processed = spark.createDataFrame(data_smote)

def output_data(data, file_name):
    output_dir = "/opt/data/" + file_name

    if os.path.exists(output_dir):
        shutil.rmtree(output_dir)

    data.coalesce(1).write.option("header", "true").csv(output_dir)

    final_file = os.path.join(base_path, f"{file_name}.csv")

    for file in os.listdir(output_dir):
        if file.startswith("part-"):
            shutil.move(os.path.join(output_dir, file), final_file)

    shutil.rmtree(output_dir)

output_data(data_train_processed, train_file)
output_data(data_test_processed, test_file)
