import logging
from pyspark.sql import SparkSession
from pyspark.ml.feature import VectorAssembler
from pyspark.ml.classification import RandomForestClassifier
from pyspark.ml.evaluation import MulticlassClassificationEvaluator
from mlflow.models import infer_signature
import mlflow
from pyspark.ml import Pipeline
import glob

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

mlflow.set_tracking_uri("http://mlflow_server:5000")

spark = SparkSession.builder \
    .appName("Preprocessing") \
    .getOrCreate()

base_path = "/opt/data/versioning/"
base_train_name = "train_data"
base_test_name = "test_data"

def get_highest_version(base_path, base_name):
    existing_files = glob.glob(f"{base_path}{base_name}*.csv")
    
    version_numbers = []
    for file in existing_files:
        parts = file.replace(base_path, "").replace(".csv", "").replace(base_name, "")
        if parts.isdigit():
            version_numbers.append(int(parts))
    
    return max(version_numbers, default=1)

highest_version = get_highest_version(base_path, base_train_name)

train_file = f"{base_path}{base_train_name}{highest_version}.csv"
test_file = f"{base_path}{base_test_name}{highest_version}.csv"

df_train = spark.read.csv(train_file, header=True, inferSchema=True)
df_test = spark.read.csv(test_file, header=True, inferSchema=True)
all_columns = df_train.columns
feature_columns = [col for col in all_columns if col != "churn"]
assembler = VectorAssembler(inputCols=feature_columns, outputCol="features")
rf = RandomForestClassifier(labelCol="churn", featuresCol="features", numTrees=10)
pipeline = Pipeline(stages=[assembler, rf])

model = pipeline.fit(df_train)
predictions = model.transform(df_test)

evaluator = MulticlassClassificationEvaluator(
    labelCol="churn", predictionCol="prediction", metricName="accuracy")
accuracy = evaluator.evaluate(predictions)
logger.info("Accuracy = %g" % (accuracy))

predictions = predictions.select("prediction")
input_example = df_test.drop("churn")
signature = infer_signature(input_example, predictions)

import os

if not os.path.exists("/opt/data/mlflow-tmp"):
    os.makedirs("/opt/data/mlflow-tmp")

with mlflow.start_run() as run:
    model_info = mlflow.spark.log_model(
        model,
        "Random Forest Model",
        signature=signature,
        registered_model_name="Random Forest Model",
        dfs_tmpdir="/opt/data/mlflow-tmp"
    )
    mlflow.log_metric("accuracy", accuracy)
    mlflow.log_param("numTrees", 10)
